package calistay;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import junit.framework.TestCase;
import tr.gov.tubitak.uekae.esya.api.asn.x509.ECertificate;
import tr.gov.tubitak.uekae.esya.api.cmssignature.ISignable;
import tr.gov.tubitak.uekae.esya.api.cmssignature.SignableByteArray;
import tr.gov.tubitak.uekae.esya.api.cmssignature.attribute.EParameters;
import tr.gov.tubitak.uekae.esya.api.cmssignature.example.util.SmartCardManager;
import tr.gov.tubitak.uekae.esya.api.cmssignature.signature.BaseSignedData;
import tr.gov.tubitak.uekae.esya.api.cmssignature.signature.ESignatureType;
import tr.gov.tubitak.uekae.esya.api.cmssignature.signature.Signer;
import tr.gov.tubitak.uekae.esya.api.common.crypto.BaseSigner;
import tr.gov.tubitak.uekae.esya.api.crypto.alg.DigestAlg;
import tr.gov.tubitak.uekae.esya.api.infra.tsclient.TSSettings;
import tr.gov.tubitak.uekae.esya.asn.util.AsnIO;

public class ImzaAtma extends TestCase 
{
	public void  testImzaConvert()throws Exception
	{
		BaseSignedData bs = new BaseSignedData(getSign());

		TSSettings tsSettings = new TSSettings("http://zd.kamusm.gov.tr", 1580, "Ft72!y9a");

		Map<String,Object> params = new HashMap<String, Object>();
		params.put(EParameters.P_TS_DIGEST_ALG, DigestAlg.SHA1);
		params.put(EParameters.P_TSS_INFO, tsSettings );
		params.put(EParameters.P_VALIDATE_CERTIFICATE_BEFORE_SIGNING, false);

		bs.getSignerList().get(0).convert(ESignatureType.TYPE_EST, params);
		
		byte [] converted = bs.getEncoded();
		
		AsnIO.dosyayaz(converted, "testdata/Donusturuldu.p7s");

	}
	
	
	
	public void testESTImza() throws Exception
	{
		BaseSignedData bs = new BaseSignedData();

		bs.addContent(getSimpleContent()); // Bütünleşik imza
		//bs.addContent(getSimpleContent(), false); //Ayrık imza


		HashMap<String, Object> params = new HashMap<String, Object>();
		params.put(EParameters.P_VALIDATE_CERTIFICATE_BEFORE_SIGNING, false);
		
		TSSettings tsSettings = new TSSettings("http://zd.kamusm.gov.tr", 1580, "Ft72!y9a");

		params.put(EParameters.P_TSS_INFO, tsSettings);
		params.put(EParameters.P_TS_DIGEST_ALG, DigestAlg.SHA1);



		bs.addSigner(ESignatureType.TYPE_EST, getSignerCertificate(), 
				getSignerInterface(), null, params);

		byte []signature = bs.getEncoded();
		
		AsnIO.dosyayaz(signature, "testdata/ESTImzaAtma.p7s");
		
		System.out.println("Bitti");

	}
	
	
	public void testSeriImzaEkle() throws Exception
	{
		BaseSignedData bs = new BaseSignedData(getSign());

		HashMap<String, Object> params = new HashMap<String, Object>();
		params.put(EParameters.P_VALIDATE_CERTIFICATE_BEFORE_SIGNING, false);

		List<Signer> signerList =  bs.getSignerList();
		Signer lastSigner = null;

		if(signerList!=null && signerList.size() > 0)
		{
			lastSigner = signerList.get(signerList.size() - 1);
			while(lastSigner.getCounterSigners().size() > 0)
				lastSigner = lastSigner.getCounterSigners(). get(lastSigner.getCounterSigners().size()-1);
		}

		lastSigner.addCounterSigner(ESignatureType.TYPE_BES, getSignerCertificate(), getSignerInterface(), null, params);
		
		byte [] signature = bs.getEncoded();
		
		AsnIO.dosyayaz(signature, "testdata/seriImza.p7s");

	}
	
	public void testImzaEkle() throws Exception
	{
		BaseSignedData bs = new BaseSignedData(getSign());

		HashMap<String, Object> params = new HashMap<String, Object>();
		params.put(EParameters.P_VALIDATE_CERTIFICATE_BEFORE_SIGNING, false);

		//Harici İmza
		//File file = new File(movieFile);
		//ISignable externalContent = new SignableFile(file);
		//params.put(EParameters.P_EXTERNAL_CONTENT, externalContent);

		bs.addSigner(ESignatureType.TYPE_BES, getSignerCertificate(), 
				getSignerInterface(), null, params);

		byte []signature = bs.getEncoded();
		
		AsnIO.dosyayaz(signature, "testdata/ImzaEklendi.p7s");

	}
	
	private byte [] getSign() throws Exception
	{
		return AsnIO.dosyadanOKU("testdata/ImzaAtma.p7s");
	
	}

	public void testImza() throws Exception
	{
		BaseSignedData bs = new BaseSignedData();

		bs.addContent(getSimpleContent()); // Bütünleşik imza
		//bs.addContent(getSimpleContent(), false); //Ayrık imza


		HashMap<String, Object> params = new HashMap<String, Object>();
		params.put(EParameters.P_VALIDATE_CERTIFICATE_BEFORE_SIGNING, false);


		bs.addSigner(ESignatureType.TYPE_BES, getSignerCertificate(), 
				getSignerInterface(), null, params);

		byte []signature = bs.getEncoded();
		
		AsnIO.dosyayaz(signature, "testdata/ImzaAtma.p7s");

	}

	private BaseSigner getSignerInterface() throws Exception
	{
		return SmartCardManager.getInstance().getSigner("12345", getSignerCertificate());
	}

	private ECertificate getSignerCertificate() throws Exception
	{
		return SmartCardManager.getInstance().getSignatureCertificate(true, false);
	}

	private ISignable getSimpleContent() 
	{
		return new SignableByteArray("test".getBytes());
	}
}
